##################################################
##### Set Variables and Source File Location #####
##################################################
# Step 0: Install R and R Studio if you haven't already. This is free, open source software that can be retrieved at the following URLs.
# https://www.r-project.org/
# https://www.rstudio.com/

#Step 1: identify the directory where you want to save the resulting outputs. Note that the direction of the slash (/) matters.
setwd("C:/File/Location/You/Want/To/Save/Files/To/")

# Step 2: identify where you saved the example file. I recommend you save it in the same directory to which you point in step 1.
DisproMaster <- read.csv("C:/File/Location/Of/II21 Tool Showcase WI example.csv", stringsAsFactors = F)

# Step 3: Install required package (if not already installed).
install.packages("tidyverse")

# Step 4 (OPTIONAL!): Set the variables for your state. These are preset to Wisconsin's criteria, 
## and the example file provided is designed to show sample reports based on this criteria

MinNumerator <- 10 # minimums to be considered dispro and for comparison group (if comparison group doesn't meet minimums, will use alternate risk ratio)
MinDenominator <- 20
RiskRatio <- 2 # Wisconsin uses the same risk ratio for all categories, but each category could be run seperately to accommodate varying risk ratio minimums
ConsecutiveYears <- 3 # number of consecutive years needed to be identified as Dispro
ProgressMinimum <- 0.25 # decline in Risk Ratio that prevents dispro determination if met for 2 consecutive years
ReportingYear <- 2020 # Source year for the determination. Wisconsin includes ReportingYear+1 data in our calculations for indicators that use Child Count

# Note: in the provided example, only two districts are included. but the code below will accommodate calculating statewide rates for 
## the alternate risk ratio if it is needed in a real-world setting.

# Step 5: run the script! it will create the necessary file outputs needed for rendering reports. when complete, move on to "Sample Racial Equity Render.R"

library(dplyr)

########################
##### Calculations #####
########################

DisproMaster$GroupRisk <- with(DisproMaster, GroupCount / BaseCount)

# Counts by District & Group, but not Race
DisproMaster <- DisproMaster %>% group_by(DCODE, REPORTING_YEAR, CATEGORY_CODE, GROUP_CODE) %>%
  mutate(DistrictGroupCount = sum(GroupCount, na.rm = T),
         DistrictBaseCount = sum(BaseCount, na.rm = T),
         DistrictGroupRisk = DistrictGroupCount / DistrictBaseCount)

# Counts by Group & Race, but not District
DisproMaster <- DisproMaster %>% group_by(REPORTING_YEAR, CATEGORY_CODE, GROUP_CODE, RACE) %>%
  mutate(StateRaceGroupCount = sum(GroupCount, na.rm = T),
         StateRaceBaseCount = sum(BaseCount, na.rm = T),
         StateRaceGroupRisk = StateRaceGroupCount/StateRaceBaseCount)

# StateGroupCount Counts by Group, but not District or Race
DisproMaster <- DisproMaster %>% group_by(REPORTING_YEAR, CATEGORY_CODE, GROUP_CODE) %>%
  mutate(StateGroupCount = sum(GroupCount, na.rm = T),
         StateBaseCount = sum(BaseCount, na.rm = T),
         StateGroupRisk = StateGroupCount/StateBaseCount)
DisproMaster$RatioType <- with(DisproMaster, ifelse(DistrictGroupCount - GroupCount >= MinNumerator &
                                                      DistrictBaseCount - BaseCount >= MinDenominator, 'R', 'A'))
DisproMaster$RegAltRatio <- with(DisproMaster,
                                 ifelse(RatioType == 'R', GroupRisk / ((DistrictGroupCount-GroupCount)/(DistrictBaseCount-BaseCount)),
                                        GroupRisk / ((StateGroupCount - StateRaceGroupCount) / (StateBaseCount - StateRaceBaseCount))))

# check for progress
DisproMaster <- DisproMaster %>% group_by(CATEGORY_CODE, GROUP_CODE, DCODE, RACE) %>%
  mutate(Progress = ifelse(lag(RegAltRatio, order_by = REPORTING_YEAR) - RegAltRatio > ProgressMinimum, T, F))

# Check for 3 consecutive years #
DisproDeterms <- DisproMaster %>% group_by(DCODE, CATEGORY_CODE, GROUP_CODE, RACE) %>%
  summarise(Active = length(DCODE[REPORTING_YEAR == ReportingYear]),
            YearsData = length(DCODE),
            YearsCell = length(DCODE[GroupCount > MinNumerator & BaseCount > MinDenominator]),
            YearsRisk = length(DCODE[RegAltRatio >= RiskRatio]),
            YearsProgress = sum(Progress==T, na.rm = T),
            Dispro = ifelse(YearsCell == ConsecutiveYears & YearsRisk == ConsecutiveYears & YearsProgress < 2, T, F))

##### merge #####
DisproDistricts <- subset(DisproDeterms, Dispro == T,
                                select = c(DCODE, DNAME, CATEGORY_CODE, GROUP_CODE, RACE))

# Save copies
write.csv(DisproMaster, file = paste0(ReportingYear, " Sample DisproMaster.csv"), row.names = F)
write.csv(DisproDeterms, file = paste0(ReportingYear, " Sample DisproDeterms.csv"), row.names = F)
write.csv(DisproDistricts, file = paste0(ReportingYear, " Sample DisproDistricts.csv"), row.names = F)